local MAJOR, MINOR = "LibFail-1.0", "3"
local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

local callbacks = LibStub("CallbackHandler-1.0"):New(lib)

lib.frame = CreateFrame("Frame")

local frame = lib.frame
frame:RegisterEvent("PLAYER_ENTERING_WORLD")

--[===[@debug@ 
local fail_events = {
	"Fail_Sartheron_LavaWaves",
	"Fail_Heigan_Dance",
	"Fail_Any_VoidZone",
	"Fail_Sapphiron_Breath",
	"Fail_Frogger",
	"Fail_Thaddius_PolaritySwitch",
	"Fail_Thaddius_Jump",
}


function lib:Test()
    local e = math.floor(math.random() * #fail_events) + 1
    local p = math.floor(math.random() * 5) + 1

    callbacks:Fire(fail_events[e], "Test"..p)
end

--@end-debug@]===]

function lib:GoInactive()
    if not self.active then return end

	self.LastTsunami = {}
	self.ChargeCounter = {}
	self.LastCharge = {}
	self.ThaddiusAlive = true;
	self.DeathTime = 0
	self.LastSlime = {}
    self.LastFroggerDmg = {}

    self.active = nil

    frame:RegisterEvent("RAID_ROSTER_UPDATE")
	frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

    callbacks:Fire("Fail_Inactive")
end

lib.active = true
lib:GoInactive()

lib.THADDIUS_JUMP_WINDOW = 120
lib.THADDIUS_JUMP_RETRY_WINDOW = 5
lib.FROGGER_DEATH_WINDOW = 4

do
	local _, etype, f
	
	frame:SetScript("OnEvent", function (self, event, ...)
		if event == "COMBAT_LOG_EVENT_UNFILTERED" then
			_, etype = ...
			f = lib[etype]
	
			if f then
				f(lib, ...)
			end
			
			return
		end

		f = lib[event]

		if f then
			f(lib, ...)
		end
	end)
end


function lib:GoActive()
    if self.active then return end

    --frame:UnregisterEvent("RAID_ROSTER_UPDATE")
	frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

	self.active = true

    callbacks:Fire("Fail_Active")
end


function lib:PLAYER_ENTERING_WORLD(...)
	if GetNumRaidMembers() > 0 then
        self:GoActive()
	else
        self:GoInactive()
	end
end

function lib:RAID_ROSTER_UPDATE(...)
	if GetNumRaidMembers() > 0 then
        self:GoActive()
	else
        self:GoInactive()
	end
end

function lib:SPELL_DAMAGE(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if (sourceName=="Mirror Image") or (destName=="Mirror Image") then
		return
	end

	-- Lava Waves: Flame Tsunami 57491
	if spellId == 57491 then
		if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
			if self.LastTsunami[destName] == nil then
				self.LastTsunami[destName] = timestamp
				
				callbacks:Fire("Fail_Sartheron_LavaWaves", destName)
			else
				if (timestamp - lib.LastTsunami[destName]) > 10 then
					callbacks:Fire("Fail_Sartheron_LavaWaves", destName)
				end
				self.LastTsunami[destName] = timestamp
			end
		end

        return
	end

	-- Heigan Dance, Void Zones, Sapphiron's Breath, Frogger

    -- Frogger: Living Poision -> Explode 28433
    -- Heigan: Eruption 29371
    -- Sapphiron: Frost Breath 28524
    -- Void Blast 27812
    local FailType
--	if (spellName=="Eruption") or (spellName=="Void Blast") or (spellName=="Frost Breath") or (spellName=="Explode") then
--		if (sourceName:find("Fissure")) or (sourceName=="Sapphiron") or (sourceName=="Living Poison") then


	if spellId == 29371 then
		FailType = "Heigan_Dance"
	elseif spellId == 27812 then
		FailType = "Any_VoidZone"
	elseif spellId == 28524 then
		FailType = "Sapphiron_Breath"
	elseif spellId == 28433 then
        self.FroggerTime[destName] = timerstamp
        return
	end


	if FailType and bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
		callbacks:Fire("Fail_"..FailType, destName)
        return
	end

--		end
--	end

	-- Thaddius Polarity Switch
    -- 28062 Positive Charge
    -- 28085 Negative Charge
	if self.ThaddiusAlive then
		if spellId == 28062  or spellId == 28085 then
			if self.ChargeCounter[sourceName] == nil then
				self.ChargeCounter[sourceName] = 1
				self.LastCharge[sourceName] = timestamp
			elseif (timestamp - lib.LastCharge[sourceName]) < 2 then
				self.ChargeCounter[sourceName] = lib.ChargeCounter[sourceName] + 1
				self.LastCharge[sourceName] = timestamp
			else
				self.ChargeCounter[sourceName] = 1
				self.LastCharge[sourceName] = timestamp
			end

			if self.ChargeCounter[sourceName] == 3 then
				callbacks:Fire("Fail_Thaddius_PolaritySwitch", sourceName)
			end
		end
	end
end


function lib:ENVIRONMENTAL_DAMAGE(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, dmgType)
    if (timestamp - self.DeathTime) < self.THADDIUS_JUMP_WINDOW and dmgType == "FALLING" then 
    	if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then    
            self.LastSlime[destName] = timestamp
            callbacks:Fire("Fail_Thaddius_Jump", destName)
        end
    end
end

-- 28089 Polarity Shift 
function lib:SPELL_CAST_START(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
--	if sourceName=="Thaddius" then
		if spellId == 28089 then
			wipe(self.ChargeCounter)
			self.ThaddiusAlive = true;
		end
--	end
end

function lib:UNIT_DIED(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags)
	if (destName=="Thaddius") then
		self.ThaddiusAlive = false;
	elseif (destName=="Stalagg") or (destName=="Feugen") then
		self.DeathTime = timestamp
    elseif self.FroggerTime[destName] then
        if timestamp - self.FroggerTime[destName] < self.FROGGER_DEATH_WINDOW then
            callbacks:Fire("Fail_Frogger", destName)
        end

        self.FroggerTime[destName] = nil
	end
end

-- 28801 Slime (Under Thaddius)
function lib:SPELL_AURA_APPLIED(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if spellId == 28801 then
		if (timestamp - self.DeathTime) < self.THADDIUS_JUMP_WINDOW then -- error
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				if self.LastSlime[destName] == nil then
					callbacks:Fire("Fail_Thaddius_Jump", destName)
				elseif (timestamp - self.LastSlime[destName]) > self.THADDIUS_JUMP_RETRY_WINDOW then
					callbacks:Fire("Fail_Thaddius_Jump", destName)
				end
			end
		end
	end
end


-- 28801 Slime (Under Thaddius)
function lib:SPELL_AURA_REMOVED(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if spellId == 28801 then
		if (timestamp - self.DeathTime) < self.THADDIUS_JUMP_WINDOW then --error
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				self.LastSlime[destName] = timestamp
			end
		end
	end
end



